/*---------------------------------------------------
                  A <-- ADJ2SL(n,s,N,S,L)

Adjacency for 2-space for a stripe and a line.

Input:
   n   is the number of projection factors.
   s   is the current signature.
   N   is the numbers of unconnnetec multiplicities.
   S   is the information about a strip to be considered.
   L   is the information about a line to be considered

Output:
   A is the adjacency info.

Remark:
   Until the completeness is proved, we check for multiple
   possibilities.
----------------------------------------------------*/
#include "saclib.h"
#include "qepcad.h"

Word ADJ2SL(n,s,N,S,L)
       Word n,s,N,S,L;
{
       Word A,Ap,sp,Np,Sp,Lp,a,h;
       Word found;

Step1: /* Trivial cases. */
       if (L == NIL && S == NIL) return(NIL);
       if (L == NIL && S != NIL) return(0);

Step2: /* General case. */
       found = 0;
       h = FIRST(FIRST(L));
       for (a = h; a >= 0; a = a - 2)
         {
         if (!CHKCONS(n,s,N,S,L,a,&sp,&Np,&Sp,&Lp)) continue;
         Ap = ADJ2SL(n,sp,Np,Sp,Lp);
         if (Ap != 0) 
           {
           if (found)
             {
             SWRITE("\n############ ERROR ERROR in ");
             CLOUT(GVMCOMM);
             SWRITE("\n");
             SWRITE("Ap1 = ");
             OWRITE(RED(A));
             SWRITE("Ap2 = ");
             OWRITE(Ap);
             FAIL("ADJ2SL","Multiple configuaration.\n");
             }
           A = COMP(a,Ap); 
           found = 1;
           }
         }
       if (found)
         return(A);
       else
         return(0);
}





