/*======================================================================
                      ATOMFRDR(V; F,t)

Atomic Formula Read, robust.

\Input
  \parm{V} is a non-null list of variables. 

\Output  
  \parm{F} is an atomic formula whose variables 
           all occur in $V$, read from the input stream.
  \parm{t} is 1 if successful, 0 otherwise.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void ATOMFRDR(V, F_,t_)
       Word V, *F_,*t_;
{
       Word F,P,P1,P2,R,a,r,s,t;
       /* hide r,s,t; */

Step1: /* Read the left polynomial. */
        t = 1; r = LENGTH(V);
        RPEXPREAD(r,V,&P1,&t); if (t == 0) goto Return;

Step2: /* Read the relational operator. */
        RLOPRDR(&R,&t); if (t == 0) goto Return;

Step3: /* Read the right polynomial. */
        RPEXPREAD(r,V,&P2,&t); if (t == 0) goto Return;

Step4: /* Finish up. */
        P = RPDIF(r,P1,P2); IPSRP(r,P,&a,&P);
        s = RNSIGN(a); if (s < 0) P = IPNEG(r,P);
        F = LIST4(R,P,r,NIL); goto Return;

Step5: /* Error exit. */
        DIELOC(); t = 0; goto Return;

Return: /* Prepare for return. */
       *F_ = F;
       *t_ = t;
       return;
}
