/*======================================================================
                      FPSIGTBL(c,f,T_t,T_f; Tp_t,Tp_f)

Full Partial-Reverse Signature Tables.

\Input
  \parm{c}  is a cell of the final partial CAD.
  \parm{f}  is the number of the free variables.
  \parm{Tt} is the list of signatures for  solution cells
            visited so far.
  \parm{Tf} is the list of signatures for  non-solution cells
            visited so far.

\Ouput
  \parm{T't} is the list of signatures obtained from $T_t$ by appending
             the signatures of the  solution cells over $c$.
  \parm{T'f} is the list of signatures obtained from $T_f$ by appending
             the signatures of the  non-solution cells over $c$.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void FPSIGTBL(c,f,T_t,T_f, Tp_t_,Tp_f_)
       Word c,f,T_t,T_f, *Tp_t_,*Tp_f_;
{
       Word S,S1,Sp,Tp_f,Tp_t,cb,cp,i,j,k,m;
       /* hide i,j,m; */

Step1: /* Leaf. */
       Tp_t = T_t; Tp_f = T_f;
       cb = LELTI(c,CHILD); if (cb != NIL) goto Step2;
       k = LELTI(c,LEVEL);
       Sp = LELTI(c,SIGNPF);
       Sp = CINV(Sp);
       S = NIL;
       while (Sp != NIL)
         {
         ADV(Sp,&S1,&Sp);
         S = CCONC(S1,S);
         }       
       for (i = k + 1; i <= f; i++)
         {
         m = LENGTH(LELTI(GVPF,i));
         for (j = 1; j <= m; j++) S = COMP(UNDET,S);
         }
       if (LELTI(c,TRUTH) == TRUE)
         Tp_t = COMP(S,Tp_t);
       else
         Tp_f = COMP(S,Tp_f);
       goto Return;

Step2: /* Non--leaf. */
       while (cb != NIL)
         {
         ADV(cb,&cp,&cb);
         FPSIGTBL(cp,f,Tp_t,Tp_f,&Tp_t,&Tp_f);
         }
       goto Return;

Return: /* Prepare for return. */
       *Tp_t_ = Tp_t;
       *Tp_f_ = Tp_f;
       return;
}


