/*======================================================================
                      GENPIMP(T_t,T_f,L_t,L_f,W,T,M; L);


Generate Prime Implicants.

\Input
  \parm{Tt} is a sum-of-product for the TRUE-set of a logic function.
  \parm{Tf} is a sum-of-proudct for the FALSE-set of the logic function.
  \parm{Lt} is the length of Tt.
  \parm{Lf} is the length of Tf.
  \parm{W}  is the length of a product.
  \parm{T}  is an array to hold the products. 
  \parm{M}  is an array to hold the marks.   

\Output
  \parm{T} is filled with prime implicants to capture Tt.
  \parm{M} is filled with marks associated with prime implicants.
  \parm{L} is the number of prime implicants in T.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void GENPIMP(T_t,T_f,L_t,L_f,W,T,M, &L_)
       Word L_t,L_f,W,*L_;
       RelOpsTable T_t,T_f,T;
       MarksTable M; 
{
       Word L,i,j,t,Rem;
       Marks m;
       RelOps v;
       Word S_r,S_s,R_c,R_s,q;

Step1: /* Setup. */
       m = MARKALLOC(L_t);  
       v = RELALLOC(W);
       L = 0;
       Rem = L_t;
       for (i = 1; i <= L_t; i++)
         T_t[i][0] = 0;
       FIRSTPRD(W,&v,&S_r,&S_s,&R_c,&R_s);

Step2: /* Get the next product. */
       NEXTPRD(W,v,S_r,S_s,R_c,R_s,&v,&S_r,&S_s,&R_c,&R_s,&q);
                                /*Int*/ NMGENPRD = NMGENPRD + 1;
                                /*Int*/ if (TCPROD == 'y') TRPROD(v,W);

Step3: /* Does it capture any minterm in $H_f$? */
       for (j = 1; j <= L_f; j++)
         if (CAPTURE(v,T_f[j],W)) goto Step2;

Step4: /* Does it capture any minterm in $H_t$? */
       t = 0;
       for (j = 1; j <= L_t; j++)
         if (CAPTURE(v,T_t[j],W)) 
           { 
           t = 1; 
           m[j] = 1; 
           } 
         else 
           m[j] = 0; 
       if (t == 0) goto Step2;

Step5: /* Is it prime? */
       for (i = 1; i <= L; i++)
         if (COVER(M[i],m,L_t)) goto Step2;

Step6: /* Register the product. */
       L++;
       M[L] = MARKALLOC(L_t);
       for (j = 1; j <= L_t; j++) M[L][j] = m[j];
       T[L] = RELALLOC(W);
       for (j = 1; j <= W;   j++) T[L][j] = v[j];
                                 /*Int*/ NMGENIMP = NMGENIMP + 1;
                                 /*Int*/ if (TCPIMP == 'y') TRPIMP(v,W,m,L_t);

Step7: /* Check if there are more minterms to capture. */ 
       for (j = 1; j <= L_t;  j++)
         if (m[j] == 1 && T_t[j][0] == 0) { Rem--; T_t[j][0] = 1; }
       if (Rem > 0) goto Step2;

Return: /* Prepare for return. */
       *L_ = L;
       return;
}




