/*======================================================================
                      F <- QFFFSOP(H,L,W,P,f)

Quantifier-Free Formula From Sum-Of-Product.

\Input
  \parm{H} is a sum-of-product.
  \parm{L} is the length of H.
  \parm{W} is the length of product.
  \parm{P} is a list $(P_1,\ldots,P_r)$ where
           $P_i$ is the list of the $i$--level projection factors.
  \parm{f} is the number of free variables.
  
\Output
  \parm{F} is the qunatifier-free formula corresponding to $H$.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

Word QFFFSOP(H,L,W,P,f)
       Word L,W,P,f;
       RelOpsTable H;
{
       Word A,F,F1,F11,I1,Ip,Is,L,P1,P11,Pp,Ps,i,j,r1,rp,rs;

Step1: /* Setup. */
       Ps = NIL; rs = NIL; Is = NIL;
       Pp = P;
       while (Pp != NIL)
         {
         ADV(Pp,&P1,&Pp);
         while (P1 != NIL)
           {
           ADV(P1,&P11,&P1);
           A = LELTI(P11,PO_POLY);
           L = LELTI(P11,PO_LABEL);
           Ps = COMP(A,Ps);
           rs = COMP(SECOND(L),rs);
           Is = COMP(RED(L),Is);
           }
         }
       Ps = INV(Ps); rs = INV(rs); Is = INV(Is);

Step2: /* Convert. */
       F = LIST1(OROP);
       for (i = 1; i <= L; i++)
         {
         F1 = LIST1(ANDOP); Pp = Ps; rp = rs; Ip = Is;
         for (j = 1; j <= W; j++)
           {
           ADV(Pp,&P1,&Pp); ADV(rp,&r1,&rp); ADV(Ip,&I1,&Ip);
           if (H[i][j] != TAOP)
             {
             F11 = LIST4(H[i][j],P1,r1,I1);
             F1 = COMP(F11,F1);
             }
           }
         if (LENGTH(F1) == 2)
           F1 = FIRST(F1);
         else
           F1 = INV(F1);
         F = COMP(F1,F);
         }
       if (LENGTH(F) == 2)
         F = FIRST(F);
       else
         F = INV(F);

Return: /* Prepare for return. */
       return(F);
}




