/*======================================================================
                      Fh <- REPLACE(f,F,Fp)

Replace $F'$ into $F$ and simplify.

\Input
  \parm{f}  is the number of free variables in the input formula.
  \parm{F}  is the normalized quantifier-free part of the input formula.
  \parm{F'} is a quantifier-free formula
            produced by the algorithm~\an{SIMPLIFY}.

\Output
  \parm{F} is equivalent to a formula obtained from $F$
           by replacing each atomic formula with more than $f$ variables
           with the formula~$F'$.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

Word REPLACE(f,F,Fp)
       Word f,F,Fp;
{
       Word Fh,G,H;

Step1: /* Preprocess $F$. */
       SALGF(f,F,&G,&H); G = RMCAON(G); H = RMCAON(H);

Step2: /* Replace. */
       Fh = NQFFOR(NQFFAND(Fp,G),H); Fh = RMCAON(Fh);

Return: /* Prepare for return. */
       return(Fh);
}
