/*======================================================================
                      SALGF(f,F; G,H)

Separate Atomic formulas of Level Greater than $f$.

\Input
  \parm{f} is the number of free variables in the input formula, $f > 0$.
  \parm{F} is the non-constant normalized quantifier-free part 
           of the input formula.

\Output
  \parm{G} is a normalized quantifier-free formula.
  \parm{H} is a normalized quantifier-free formula.
           Let $\hat{F}$ be the formula obtained from $F$ by replacing 
           all atomic formulas of level greater than $f$ with $F'$.
           Then $\hat{F}$ is equivalent to $(F' \land G) \lor H$.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void SALGF(f,F, G_,H_)
       Word f,F, *G_,*H_;
{
       Word F1,Fp,G,Gp,H,Hp,T;

Step1: /* Classify the formula $F$. */
       T = FIRST(F);
       if (T == OROP) goto Step3;
       if (T == ANDOP) goto Step4;

Step2: /* Non-constant normalized atomic formula. */
       if (THIRD(F) <= f)
         { G = LIST4(NEOP,0,0,NIL); H = F; }
       else
         { G = LIST4(EQOP,0,0,NIL); H = LIST4(NEOP,0,0,NIL); }
       goto Return;

Step3: /* Disjunction. */
       Fp = RED(F); ADV(Fp,&F1,&Fp); SALGF(f,F1,&G,&H);
       while (Fp != NIL)
         {
         ADV(Fp,&F1,&Fp); SALGF(f,F1,&Gp,&Hp);
         G = NQFFOR(G,Gp); H = NQFFOR(H,Hp);
         }
       goto Return;

Step4: /* Conjunction. */
       Fp = RED(F); ADV(Fp,&F1,&Fp); SALGF(f,F1,&G,&H);
       while (Fp != NIL)
         {
         ADV(Fp,&F1,&Fp); SALGF(f,F1,&Gp,&Hp);
         if (TYPEQFF(G) == FALSE) G = NQFFAND(H,Gp);
         else if (TYPEQFF(Gp) == FALSE) G = NQFFAND(G,Hp);
         else if (TYPEQFF(G) == TRUE) G = NQFFOR(Gp,Hp);
         else if (TYPEQFF(Gp) == TRUE) G = NQFFOR(G,H);
         else if (TYPEQFF(H) == FALSE) G = NQFFAND(G,NQFFOR(Gp,Hp));
         else if (TYPEQFF(Hp) == FALSE) G = NQFFAND(NQFFOR(G,H),Gp);
         else if (TYPEQFF(H) == TRUE) G = NQFFOR(NQFFAND(G,Hp),Gp);
         else if (TYPEQFF(Hp) == TRUE) G = NQFFOR(G,NQFFAND(H,Gp));
         else G = NQFFOR(NQFFAND(NQFFOR(G,H),Gp),NQFFAND(G,Hp));
         H = NQFFAND(H,Hp);
         }
       goto Return;

Return: /* Prepare for return. */
       *G_ = G;
       *H_ = H;
       return;
}
