/*======================================================================
                      SIGTBL(c,f,W,T_t,T_f,L_t,L_f; Lp_t,Lp_f)

Signature Tables.

\Input
  \parm{c}  is a cell of the final partial CAD.
  \parm{f}  is the number of the free variables.
  \parm{W}  is the number of projection factors of level <= f.
  \parm{Tt} is the array of signatures for propagation solution cells
            visited so far.
  \parm{Tf} is the array of signatures for propagation non-solution cells
            visited so far.
  \parm{Lt} is the length of $T_t$.
  \parm{Lf} is the length of $T_f$.

\Remark
  \parm{Tt} is the array of signatures obtained from $T_t$ by appending
            the signatures of the propagation solution cells over $c$.
  \parm{Tf} is the array of signatures obtained from $T_f$ by appending
            the signatures of the propagation non solution cells over $c$.
  \parm{L't} is the length of $T_t$ after appending.
  \parm{L'f} is the length of $T_f$ after appending.

======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void SIGTBL(c,f,W,T_t,T_f,L_t,L_f, Lp_t_,Lp_f_)
       Word c,f,W,L_t,L_f, *Lp_t_,*Lp_f_;
       RelOpsTable T_t,T_f;
{
       Word S1,S11,Sp,Lp_f,Lp_t,cb,cp,k,i;
       RelOps  S;

Step1: /* At the $f$-space. */
       Lp_t = L_t; Lp_f = L_f;
       k = LELTI(c,LEVEL);
       if (k < f) goto Step2;
       if (LELTI(c,HOWTV) == BYPRP)
         {
         Sp = LELTI(c,SIGNPF);
         S = RELALLOC(W);
         i = 0;
         while (Sp != NIL)
           {
           ADV(Sp,&S1,&Sp);
           while (S1 != NIL) 
             {
             ADV(S1,&S11,&S1); i++;
             S[i] = RELFSIGN(S11);
             }
	   }
         if (LELTI(c,TRUTH) == TRUE) 
           {
           Lp_t = Lp_t + 1;
           T_t[Lp_t] = S;
	   }
         else
           {
           Lp_f = Lp_f + 1;
           T_f[Lp_f] = S;
	   }
         }
       goto Return;

Step2: /* Other cases. */
       cb = LELTI(c,CHILD);
       while (cb != NIL)
         {
         ADV(cb,&cp,&cb);
         SIGTBL(cp,f,W,T_t,T_f,Lp_t,Lp_f,&Lp_t,&Lp_f);
         }

Return: /* Prepare for return. */
       *Lp_t_ = Lp_t;
       *Lp_f_ = Lp_f;
       return;
}
