/*======================================================================
                      EXTRACTS(F,A; Ap)

Extract polynomials from a quantifier--free formula, Subalgorithm.
 
\Input
  \parm{F} is a non--constant normalized quantifier-free formula.
  \parm{A} is a list~$(A_1,\ldots,A_r)$ where each $A_i$ is
           the list of all the $i$--level polynomials found
           util the call to this algorithm was made.
\Output
  \parm{A'} is the list~$(A'_1,\ldots,A'_r)$ where each $A'_i$ is
           obtained by appending to $A_i$ all the $i$--level
           polynomials in $F$ if there are not there yet.

\SideEffect
   \parm{F} is modified in that each polynomial is assigned to
           an index.
   \parm{A} is modified.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void EXTRACTS(F,A, Ap_)
       Word F,A, *Ap_;
{
       Word A_k,Ap,Ap_k,As_k,As_k_i,F_i,Fp,I,P,T,i,k,n;
       /* hide i,n; */

Step1: /* Non-constant normalized atomic formula. */
        if (!ISATOMF(F)) goto Step2;
        FIRST4(F,&T,&P,&k,&I);
        A_k = LELTI(A,k);
        As_k = A_k; n = LENGTH(A_k);
        for (i = 1; i <= n; i++)
          {
          ADV(As_k,&As_k_i,&As_k);
          if (EQUAL(P,LELTI(As_k_i,PO_POLY)))
            { SLELTI(F,4,LIST2(k,i)); Ap = A; goto Return; }
          }
        SLELTI(F,4,LIST2(k,n + 1)); 
        P = MPOLY(P,LIST3(LFS("A"),k,n+1),NIL,PO_OTHER,PO_KEEP);
        Ap_k = SUFFIX(A_k,P); 
        SLELTI(A,k,Ap_k); 
        Ap = A;
        goto Return;

Step2: /* Other. */
        Ap = A; Fp = RED(F);
        while (Fp != NIL) { ADV(Fp,&F_i,&Fp); EXTRACTS(F_i,Ap,&Ap); }
        goto Return;

Return: /* Prepare for return. */
       *Ap_ = Ap;
       return;
}

