/*======================================================================
                      AFUPRBH(M,I,A;b,B)

Algebraic number field univariate polynomial root bound and homothetic 
transformation.  

Inputs
  M : an element of Z[x].  M is the minimal polynomial for a real algebraic
      number alpha.
  I : an acceptable isolating interval for alpha.
  A : an element of Z[alpha][x].

Outputs
  b : a binary rational number.  b = 2^k is a root bound for A.
  B : an element of Z(alpha)[x].  B = A(2^kx).
======================================================================*/
#include "saclib.h"

void   AFUPRBH(M,I,A, b_,B_)

       Word M,I,A;
       Word *b_,*B_;
{
       Word b,B;
       Word Is,B_1,B_2,v_1,v_2;

Step1: /* Initialize. */
       b = LIST2(1,1);
       B = A;
       v_1 = 1;
       v_2 = 1;
       Is = I;

Step2: /* Check for positive roots outside the unit circle. */
       if (v_1 != 0)
         {
         B_1 = IPTR1(2,B,2);
         AFUPVARIR(M,Is,B_1,&v_1,&Is);
         }

Step3: /* Check for negative roots outside the unit circle. */
       if (v_1 == 0 && v_2 != 0)
         {
         B_2 = IPTR1(2,IPNT(2,B,2),2);
         AFUPVARIR(M,Is,B_2,&v_2,&Is);
         }

Step4: /* Root bound test and homothetic transformation. */
       b = RNPROD(b,RNINT(2));
       B = IPBHTMV(2,B,1);
       if (v_1 == 0 && v_2 == 0)
         goto Return;
       else
         goto Step2;

Return:
       *b_ = b;
       *B_ = B;
       return;
}

