/*======================================================================
                      H <- SELPIMP(m,V)

Select Prime Implicants.

\Input
  \parm{m} is the number of the minterms to cover.
  \parm{V} is a list of 3-tuples $(v,l,T)$ 
           where $v$ is an prime implicant, $l$ is the length of $v$, and
           $T$ is the list of minterms captured by $v$.
           
\Output
  \parm{H} is a simple sum-of-product which captures all the minterms.
           (i.e. a short list of implicants  which together still 
           captures all the minterms.)
======================================================================*/
#include "saclib.h"
#include "qepcad.h"
#define ALIVE      1
#define DEAD       0

Word SELPIMP(m,V)
       Word m,V;
{
       Word C,C1,Cp,H,R,R1,Rp,Rp1,Rs,Rs1,T,Th,Th1,Tp,Ts,V1,Vh,Vp,Vp1,
            Vs,Vs1,c,f,i,k,l,lh,lp,ls,n,q,r,rh,rs,s,t,v,vh,vp,vs,z;
       /* hide c,f,i,k,n,q,r,rh,rs,s,t,z; */

Step1: /* Setup. */
       n = LENGTH(V); H = NIL; f = 1;
       C = NIL; for (i = 1; i <= m; i++) C = COMP(ALIVE,C);
       R = NIL; for (i = 1; i <= n; i++) R = COMP(ALIVE,R);

Step2: /* Collect the essential prime implicants. */
                                  /*Int*/ if (TCPIMPTBL == 'y') TRPIMPTBL(C,R,V);
       c = 0; Cp = C; t = 0;
       while (Cp != NIL)
         {
         c = c + 1; ADV(Cp,&C1,&Cp);
         if (C1 == ALIVE)
           {
           r = 0; Rp = R; Vp = V; k = 0;
           while (Rp != NIL && k <= 1)
             {
             r = r + 1; ADV(Rp,&R1,&Rp); ADV(Vp,&V1,&Vp);
             if (R1 == ALIVE)
               {
               FIRST3(V1,&v,&l,&T);
               if (INSET(T,c)) { k = k + 1; Vh = V1; rh = r; }
               }
             }
           if (k == 1)
             {
             SLELTI(R,rh,DEAD); FIRST3(Vh,&vh,&lh,&Th);
                                  /*Int*/ NMSELIMP = NMSELIMP + 1;
                                  /*Int*/ if (TCEPIMP == 'y') TREPIMP(rh,Vh);
             while (Th != NIL) { ADV(Th,&Th1,&Th); SLELTI(C,Th1,DEAD); t = 1; }
             H = COMP(vh,H);
                                  /*Int*/ if (TCPIMPTBL == 'y') TRPIMPTBL(C,R,V);
             }
           }
         }

Step3: /* Decide the next action. */
       s = 0; Cp = C;
       while (Cp != NIL && s == 0) { ADV(Cp,&C1,&Cp); if (C1 == ALIVE) s = 1; }
       if (s == 0) { H = INV(H); goto Return; }
       if (f == 1) { f = 0; goto Step4; }
       if (t == 0) goto Step5;

Step4: /* Remove covered prime implicants. */
       r = 0; Rp = R; Vp = V; t = 0;
       while (Rp != NIL)
         {
         r = r + 1; ADV(Rp,&Rp1,&Rp); ADV(Vp,&Vp1,&Vp);
         if (Rp1 == ALIVE)
           {
           FIRST3(Vp1,&vp,&lp,&Tp);
           rs = 0; Rs = R; Vs = V; z = 0;
           while (Rs != NIL && z == 0)
             {
             rs = rs + 1; ADV(Rs,&Rs1,&Rs); ADV(Vs,&Vs1,&Vs);
             if (Rs1 == ALIVE && r != rs)
               {
               FIRST3(Vs1,&vs,&ls,&Ts);
               if (ls <= lp)
                 {
                 q = 1; c = 0; Cp = C;
                 while (Cp != NIL && q == 1)
                   {
                   c = c + 1; ADV(Cp,&C1,&Cp);
                   if (C1 == ALIVE) if (INSET(Tp,c) && !INSET(Ts,c)) q = 0;
                   }
                 if (q == 1) z = 1;
                 }
               }
             }
           if (z == 1)
             {
             SLELTI(R,r,DEAD); t = 1;
             }
           }
         }
       if (t == 1) goto Step2;

Step5: /* Choose a shortest one. NOT Implemented yet. */
       SWRITE("#####Couldn't finish using essential prime implicants and covering!\n");
       SWRITE("#####Only the essential prime implicants are returned!!\n");
       H = INV(H); goto Return;

Return: /* Prepare for return. */
       return(H);
}
