/*==========================================================================
                    C <- AFUPFMRC(M,Q1,D1,C1,Q2,D2,C2)

Algebraic number field univariate polynomial from modular residue,
with comparison.

Inputs
  Q1: in Z.
  D1: a list of the prime divisors of Q1.
  C1: in Z/(Q1)[a,y], monic, a is an algebraic number.
  Q2: in Z.
  D2: a list of the prime divisors of Q2.
  C2: in Z/(Q2)[a,y], monic.

Output
  C : in Q(a)[y] or NIL.  If it exists, C is the unique element of
      Q(a)[y] such that H_Q1(C) = C1, H_Q2(C) = C2, and the numerators
      and denominators appearing in the rational coefficients of C are
      less sqrt(Q1/2) and sqrt(Q2/2).  Otherwise, C = NIL.
==========================================================================*/
#include "saclib.h"
#include "qepcad.h"

Word AFUPFMRC(Q1,D1,C1,Q2,D2,C2)
       Word Q1,D1,C1,Q2,D2,C2;
{
       Word C,C1p,C2p,d1,d2,c1,c2,c;

Step1: /* C1 = 0 \/ C2 = 0 */
       if (C1 == 0 || C2 == 0) {
	 if (C1 == C2)
	   C = 0;
	 else
	   C = NIL;
	 goto Return;
       }

Step2: /* deg(C1) /= deg(C2) */
       if (PDEG(C1) != PDEG(C2)) {
	 C = NIL;
	 goto Return;
       }

Step3: /* Initialize. */
       C = NIL;
       C1p = PRED(C1);
       C2p = PRED(C2);

Step4: /* Recover the coefficients. */
       while (C1p != 0 && C2p != 0) {
	 d1 = PDEG(C1p);
	 d2 = PDEG(C2p);
	 if (d1 != d2) {
	   C = NIL;
	   goto Return;
	 }
	 c1 = PLDCF(C1p);
	 c2 = PLDCF(C2p);
	 c = RUPFMRC(Q1,D1,c1,Q2,D2,c2);
	 if (c == NIL) {
	   C = NIL;
	   goto Return;
	 }
	 C = COMP2(c,d1,C);
	 C1p = PRED(C1p);
	 C2p = PRED(C2p);
       }
       C = INV(C);
       c = PMON(RNINT(1),0);
       C = COMP2(PDEG(C1),c,C);

Step5: /* Convert representation. */
       C = AFPCR(1,C);

Return: /* Prepare for return. */
       return(C);
}
