/*======================================================================
                      EC1(c,L,Bs)

Establish Children of the root.

\Input
  \parm{c} is  the root cell.
  \parm{L} is  a list of real roots. (See CONSTRUCT for detail.)
  \parm{Bs} is  the basis.

\Output
  The children of the cell \v{c} are established.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void EC1(c,L,Bs)
       Word c,L,Bs;
{
       Word B,I,J,Lp,M,N,S,Sp,P,a,b,kp,l,r,rp,s,xb,xp;
       /* hide kp,xp; */

Step1: /* Initialize. */
        S = NIL; Lp = L; kp = 1; xp = 0;
        M = PMON(1,1); J = LIST2(0,0); 

Step2: /* No real root. */
        if (Lp != NIL) goto Step3;
        a = 0;
        b = LIST1(a); s = LIST3(M,J,b);
        xp = xp + 1; xb = LIST1(xp);
        P = LIST1(0);
        Sp = MCELL(kp,NIL,FALSE,UNDET,s,xb,P,UNDET,NIL,NIL); S = COMP(Sp,S);
        SLELTI(c,CHILD,S);
        goto Return;

Step3: /* First sector. */
        ADV2(Lp,&I,&B,&Lp); FIRST2(I,&l,&r);
        if (EQUAL(l,r))
          a = AFFRN(RNDIF(l,RNINT(1)));
        else
          a = AFFRN(l);
        b = LIST1(a); s = LIST3(M,J,b);
        xp = xp + 1; xb = LIST1(xp);
        P = LIST1(0);
        Sp = MCELL(kp,NIL,FALSE,UNDET,s,xb,P,UNDET,NIL,NIL); S = COMP(Sp,S);

Step4: /* First section. */
        if (PDEG(B) == 1)
          { a = IUPRLP(B); a = AFFRN(a); b = LIST1(a); s = LIST3(M,J,b); }
        else
          { a = AFGEN(); b = LIST1(a); s = LIST3(B,I,b); }
        xp = xp + 1; xb = LIST1(xp);
        P = LIST1(0);
        N = LIST1(LIST2(LSRCH(B,Bs),1));
        Sp = MCELL(kp,NIL,FALSE,UNDET,s,xb,P,UNDET,NIL,N); S = COMP(Sp,S);

Step5: /* Check if there are more roots. */
        if (Lp == NIL) goto Step9;
        rp = r;

Step6: /* Next sector. */
        ADV2(Lp,&I,&B,&Lp); FIRST2(I,&l,&r);
        if (EQUAL(l,r))
          a = AFFRN(RNQ(RNSUM(rp,l),RNINT(2)));
        else
          a = AFFRN(l);
        b = LIST1(a); s = LIST3(M,J,b);
        xp = xp + 1; xb = LIST1(xp);
        P = LIST1(0);
        Sp = MCELL(kp,NIL,FALSE,UNDET,s,xb,P,UNDET,NIL,NIL); S = COMP(Sp,S);

Step7: /* Next section. */
        if (PDEG(B) == 1)
          { a = IUPRLP(B); a = AFFRN(a); b = LIST1(a); s = LIST3(M,J,b); }
        else
          { a = AFGEN(); b = LIST1(a); s = LIST3(B,I,b); }
        xp = xp + 1; xb = LIST1(xp);
        P = LIST1(0);
        N = LIST1(LIST2(LSRCH(B,Bs),1));
        Sp = MCELL(kp,NIL,FALSE,UNDET,s,xb,P,UNDET,NIL,N); S = COMP(Sp,S);

Step8: /* Loop. */
        goto Step5;

Step9: /* Last sector. */
        a = AFFRN(RNSUM(r,RNINT(1)));
        b = LIST1(a); s = LIST3(M,J,b);
        xp = xp + 1; xb = LIST1(xp);
        P = LIST1(0);
        Sp = MCELL(kp,NIL,FALSE,UNDET,s,xb,P,UNDET,NIL,NIL); S = COMP(Sp,S);

Step10: /* Finalize. */
        S = INV(S); SLELTI(c,CHILD,S); goto Return;

Return: /* Prepare for return. */
       return;
}
