/*======================================================================
                      PROPAGATE(D,c,k,f,Q)

Propagate truth values.

\Input
   \parm{D} is a partial CAD.
   \parm{c} is a cell in $D$.
   \parm{k} is the level of the cell~$c$.
   \parm{f} is the number of free variables in the input formula.
   \parm{Q} is the list $(Q_{f+1},\ldots,Q_r)$  of the quantifiers
            in the input formula.

\Output
   The truth values of the cell~$c$ and some of its ancestors 
   are determined if possible.
   The descendants of each cell whose truth value
   has been determined through the propagation are removed from the
   partial CAD~$D$.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void PROPAGATE(D,c,k,f,Q)
       Word D,c,k,f,Q;
{
       Word A0,A1,E0,E1,cp,k0,kp,q;
       /* hide A0,A1,E0,E1,kp; */

Step1: /* Initialize. */
       cp = c; kp = k; k0 = LELTI(D,LEVEL);

Step2: /* Done? */
       if (kp < f  || kp < k0) goto Return;

Step3: /* All true. */
       TCHILD(cp,&A1,&A0,&E1,&E0);
       if (A1)
         { SLELTI(cp,TRUTH,TRUE); SLELTI(cp,HOWTV,BYPRP); goto Step8; }

Step4: /* All false. */
       if (A0)
         { SLELTI(cp,TRUTH,FALSE); SLELTI(cp,HOWTV,BYPRP); goto Step8; }

Step5: /* At least one true, $Q_{k'+1} = \exists$. */
       q = LELTI(Q,kp + 1 - f);
       if (E1 && (q == EXIST))
         { SLELTI(cp,TRUTH,TRUE); SLELTI(cp,HOWTV,BYPRP); goto Step8; }

Step6: /* At least one false and $Q_{k'+1} = \forall$. */
       if (E0 && (q == UNIVER))
         { SLELTI(cp,TRUTH,FALSE); SLELTI(cp,HOWTV,BYPRP); goto Step8; }

Step7: /* Fail to propagate. */
       goto Return;

Step8: /* Prune. */
       SLELTI(cp,CHILD,NIL);
       kp = kp - 1;
       cp = DESCENDANT(D,LELTI(cp,INDX),kp);
       goto Step2;

Return: /* Prepare for return. */
       return;
}



