/*======================================================================
                      t <- QFFTEV(F,c,k)

Quantifier-Free Formula Trial Evaluation.

\Input
 \parm{F} is a non-constant normalized quantifier--free formula.
 \parm{c} is a non-root cell.
 \parm{k} is the level of the cell~$c$.
  
\Output
 \parm{t} is the trial truth value of $F$ on the cell $c$.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

Word QFFTEV(F,c,k)
       Word F,c,k;
{
       Word F1,Fp,I,Pt,T,i,j,m,s,t,tp,z;
       /* hide m,t,tp,z; */

Step1: /* Classify the formula $F$. */
       T = FIRST(F);
       if (T == OROP) goto Step3;
       if (T == ANDOP) goto Step4;

Step2: /* Non-constant normalized atomic formula. */
       I = FOURTH(F); FIRST2(I,&i,&j);
       if (i > k) { t = UNDET; goto Return; }
       Pt = LELTI(LELTI(c,SIGNPF),k-i+1); 
       s = LELTI(Pt,j);
       if (s == UNDET) { t = UNDET; goto Return; }
       switch (T)
         {
         case LTOP: if (s < 0)  t = TRUE; else t = FALSE; break;
         case LEOP: if (s <= 0) t = TRUE; else t = FALSE; break;
         case GTOP: if (s > 0)  t = TRUE; else t = FALSE; break;
         case GEOP: if (s >= 0) t = TRUE; else t = FALSE; break;
         case EQOP: if (s == 0) t = TRUE; else t = FALSE; break;
         case NEOP: if (s != 0) t = TRUE; else t = FALSE; break;
         }
       goto Return;

Step3: /* Disjunction. */
       Fp = RED(F); z = 1;
       while (Fp != NIL)
         {
         ADV(Fp,&F1,&Fp);
         tp = QFFTEV(F1,c,k);
         if (tp == TRUE) { t = TRUE; goto Return; }
         if (tp == UNDET) z = 0;
         }
       if (z == 1) t = FALSE; else t = UNDET;
       goto Return;

Step4: /* Conjunction. */
       Fp = RED(F); z = 1;
       while (Fp != NIL)
         {
         ADV(Fp,&F1,&Fp);
         tp = QFFTEV(F1,c,k);
         if (tp == FALSE) { t = FALSE; goto Return; }
         if (tp == UNDET) z = 0;
         }
       if (z == 1) t = TRUE; else t = UNDET;
       goto Return;

Return: /* Prepare for return. */
       return(t);
}

