/*===========================================================================
		      C <- RUPFMRC(M1,D1,C1,M2,D2,C2)

Rational univariate polynomial from modular residue, with comparison.

Inputs
  M1 : in Z.
  D1 : a list of all the prime divisors if M1.
  C1 : in Z/(M1)[x].
  M2 : in Z.
  D2 : a list of all the prime divisors if M2.
  C2 : in Z/(M2)[x].

Output
  C : in Q[x] or NIL.  If it exists, C is the unique element of Q[x]
      such that H_M1(C) = C1, H_M2(C) = C2 and all numerators and 
      denominators appearing in the rational coefficients of C are
      less than sqrt(M1/2) and sqrt(M2/2).  Otherwise, C = NIL.
===========================================================================*/
#include "saclib.h"
#include "qepcad.h"

Word RUPFMRC(M1,D1,C1,M2,D2,C2)
       Word M1,D1,C1,M2,D2,C2;
{
       Word C,C1p,C2p,d1,d2,c1,c2,m1,m2,M1p,M2p,t,r1,r2;

Step1: /* C1 = 0 \/ C2 = 0 */
       if (C1 == 0 || C2 == 0) {
	  if (C1 == C2)
	     C = 0;
	  else
	     C = NIL;
	  goto Return; }

Step2: /* Initialize. */
       C = NIL;
       C1p = C1;
       C2p = C2;
       ISQRT(IQ(M1,2),&M1p,&t);
       ISQRT(IQ(M2,2),&M2p,&t);

Step3: /* Recover the rational coefficients, if possible. */
       while (C1p != 0 || C2p != 0) {
	  d1 = PDEG(C1p);
	  d2 = PDEG(C2p);
	  if (d1 != d2) {
	     C = NIL;
	     goto Return; }
	  c1 = PLDCF(C1p);
	  if (ISIGNF(c1) < 0)
	     c1 = ISUM(M1,c1);
	  c2 = PLDCF(C2p);
	  if (ISIGNF(c2) < 0)
	     c2 = ISUM(M2,c2);
	  m1 = ILOG2(M1);
	  r1 = RNFMR(M1,M1p,m1,c1);
	  if (r1 == NIL) {
	     C = NIL;
	     goto Return; }
	  m2 = ILOG2(M2);
	  r2 = RNFMR(M2,M2p,m2,c2);
	  if (EQUAL(r1,r2)) {
	     if (r1 != 0)
		C = COMP2(r1,d1,C); }
	  else {
	     C = NIL;
	     goto Return; }
	  C1p = PRED(C1p);
	  C2p = PRED(C2p); }
       C = INV(C);

Return: /* Prepare for return. */
       return(C);
}
