/*======================================================================
                      S <- SUBST(k,M,b,B)

Substitute the sample point into the projection factors.

\Input
  \parm{k} is a positive $\beta$--integer.
  \parm{M} is the integral minimal polynomial of an algebraic number $\alpha$.
  \parm{b} is the $k$-tuple coordinates of a sample point.
  \parm{B} is a list~$(B_1,\ldots,B_m)$ of 
           $(k+1)$--variate integral polynomials.

\Output
  \parm{S} is the list~$(S_1,\ldots,,S_m)
            where $S_i = B_i(b_1,\ldots,b_k,x_{k+1})$.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

Word SUBST(k,M,b,B)
       Word k,M,b,B;
{
       Word B1,Bp,S,S1;

Step1: /* Substitute. */
        S = NIL;
        Bp = B;
        while (Bp != NIL)
           {
           ADV(Bp,&B1,&Bp);
           S1 = IPAFME(k + 1,M,B1,b);
           S = COMP(S1,S);
           }
        S = INV(S);
        goto Return;

Return: /* Prepare for return. */
       return(S);
}
